;********************************************************

;

; CP/M PROGRAM TO COPY CON: INPUT LINES TO LPT:

;

;	The CP/M console buffer routine is used.

;

;	Control functions:

;	  CTRL-C at line beginning reboots system

;	  CTRL_E continues print line onto next line

;	  CTRL_H or BACKSPACE deletes last chatracter

;	  CTRL_J or LINE FEED ends current line

;	  CTRL_M or CARRIAGE RETURN ends current line

;	  CTRL_R redisplays line without printing

;	  CTRL_U deletes entire current line and

;		 continues on next line

;	  CTRL_X deletes current line

;	  CTRL-Z and CARRIAGE RETURN or LINE FEED

;		 terminates this program

;

;	BY ROBERT A. LINDLEY

;	21 HANCOCK ST. BEDFORD, MA 01730

;	11/22/83

;

BDOS	EQU	05	; CP/M ENTRY

BDOSA	EQU	6

CONOUT	EQU	2	; CONSOLE OUTPUT

LSTOUT	EQU	5	; LISTING OUTPUT

PRSTR	EQU	9	; PRINT STRING TO $

RDCBUF	EQU	10	; READ CONSOLE BUFFER

TESTOFF EQU	2FEH	; ^P VERIFY ADDR

LISTCP	EQU	0DH	; ^P OFFSET IN PAGE

MVIC	EQU	03EH

;

NL	EQU	10

CR	EQU	13

CTRLZ	EQU	26

ESC	EQU	27



	ORG	0100H

BEGIN:	MVI	C,PRSTR ; CLEAR THE SCREEN

	LXI	D,CLS

	CALL	BDOS

*; TRY TO TURN OFF SCREEN PRINT

	LHLD	BDOSA	; TEST FOR CP/M VER. 2.2

	LXI	D,TESTOFF ; VERIFY OFFSET INTO BDOS

	DAD	D

	LXI	D,STRING

COMP:	LDAX	D	; GET COMPARE BYTE

	ORA	A

	JZ	COMPOK	; ZERO ENDS COMPARE

	CMP	M

	INX	H	; MOVE TO NEXT BYTE

	INX	D

	JZ	COMP

	JMP	SIGNON	; WRONG VERSION, DON'T TRY

COMPOK: MVI	L,LISTCP ; CP/M VER 2.2. FOUND

	XRA	A	; TURN OFF ^P

	MOV	M,A

SIGNON: MVI	C,PRSTR ; PRINT OPENING MESSAGE

	LXI	D,MSG	; AND CLEAR SCREEN

	CALL	BDOS

*  LOOP READING AND WRITING CHARACTERS

RLOOP:	LXI	D,BUFFER

	MVI	C,RDCBUF ; READ A LINE

	CALL	BDOS

	MVI	C,CONOUT

	MVI	E,NL	; CONSOLE NEEDS A NL

	CALL	BDOS

	LXI	H,CHARS ; POINT TO CHARACTERS

	SHLD	PCHARS	; TO BE PRINTED



; LOOP TO PRINT CHARACTERS FROM BUFFER

CLOOP:	LXI	H,CURCNT ; NUMBER OF CHARACTERS

	MOV	A,M	;  REMAINING IN BUFFER

	DCR	M

	ORA	A	; TEST FOR END OF LINE

	JNZ	LOADCH

	CALL	ENDLIN

	JMP	RLOOP	; CONTINUE WITH NEXT LINE

LOADCH: CPI	1	; TEST FOR LAST CHARACTER

	LHLD	PCHARS	; DATA POINTER

	MOV	A,M	; GET NEXT CHARACTER

	CPI	CTRLZ

	JNZ	DOCHAR

	CALL	ENDLIN	; CTRL-Z IS LAST CHARACTER

; CONTROL Z FOUND - RETURN TO CP/M

	RET

;

; SEND CHARACTER TO PRINTER

DOCHAR: MOV	E,A

	INX	H

	SHLD	PCHARS	; UPDATE CHAR POINTER

	MVI	C,LSTOUT

	CALL	BDOS

	JMP	CLOOP	; CONTINUE CHARACTER LOOP

;

; SEND END OF LINE CARRIAGE RETURN AND LINE FEED

ENDLIN: MVI	C,LSTOUT

	MVI	E,CR	; SEND A CR

	CALL	BDOS

	MVI	C,LSTOUT

	MVI	E,NL	; SEND A NL

	CALL	BDOS

	RET

;

*  DATA SPACE

CLS:	DB	ESC,'E$' ; Clear screen

MSG:	DB	'What you type here will be printed '

	DB	'line by line.',CR,NL,NL

	DB	'Control functions:',CR,NL

	DB	'CTRL-C at line beginning reboots system',CR,NL

	DB	'CTRL_E continues print line onto next line',CR,NL

	DB	'CTRL_H or BACKSPACE deletes last character',CR,NL

	DB	'CTRL_J or LINE FEED prints current line',CR,NL

	DB	'CTRL_M or RETURN prints current line',CR,NL

	DB	'CTRL_R redisplays line without printing',CR,NL

	DB	'CTRL_U deletes current line and'

	DB	' continues on next line',CR,NL

	DB	'CTRL_X erases current line',CR,NL

	DB	'CTRL-Z and RETURN or LINE FEED'

	DB	' terminates this program',CR,NL,NL,'$'

STRING: DB	RET,MVIC,1,0

PCHARS: DW	0	; POINTER TO CHARACTERS IN BUFFER

BUFFER	EQU	$	; BUFFER FOR "READ CONSOLE BUFFER"

MAXCNT: DB	255	; MAXIMUM # OF CHARACTERS

CURCNT: DB	0	; CURRENT # OF CHARACTERS IN BUFFER

CHARS:	DS	256	; CHARACTERS FROM CONSOLE GO HERE

	END	BEGIN










OLE GO HERE

	END	BEGIN










